@@ -95,7 +95,7 @@ Q_SIGNALS:
 
 private:
     explicit Application();
-    ~Application() = default;
+    ~Application();
 
     Application(const Application &) = delete;
     Application &operator=(const Application &) = delete;
@@ -105,17 +105,17 @@ private:
     void setupWorkerThread();
     void setupAboutData();
     void setupCommandLineParser();
-    void registerQmlTypes();
     void setupQmlSettingsTypes();
     QAbstractItemModel *colorSchemesModel();
     QApplication *m_app{nullptr};
     KAboutData m_aboutData;
     KSharedConfig::Ptr m_config;
-    QCommandLineParser *m_parser{nullptr};
+    std::unique_ptr<QCommandLineParser> m_parser;
     QMap<int, QUrl> m_urls;
     KColorSchemeManager *m_schemes{nullptr};
     QString m_systemDefaultStyle;
     QQmlApplicationEngine *m_qmlEngine{nullptr};
+    std::unique_ptr<ApplicationEventFilter> m_appEventFilter;
 };
 
 #endif // APPLICATION_H
